package com.jz.common.utils;

import java.io.File;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.apache.commons.lang3.StringUtils;
import org.apache.commons.lang3.time.DateFormatUtils;

import com.alibaba.fastjson.JSON;
import com.jz.common.enums.NodeChangeTypeEnum;
import com.jz.dmp.modules.controller.DataIntegration.bean.flow.FlowNode;
import com.jz.dmp.modules.controller.DataIntegration.bean.flow.FlowNodeChangeInfo;
import com.jz.dmp.modules.controller.DataIntegration.bean.flow.FlowPro;
import com.jz.dmp.modules.model.DmpNavigationTree;
import com.jz.dmp.modules.model.DmpProject;
import com.jz.dmp.modules.model.DmpProjectSystemInfo;
import com.jz.dmp.modules.model.DmpWorkFlowSubmitDetails;
import com.jz.dmp.modules.service.DmpDevelopTaskService;
import com.jz.dmp.modules.service.DmpNavigationTreeService;
import com.jz.dmp.modules.service.DmpWorkFlowSubmitDetailsService;

import lombok.Data;

@Data
public class FlowParseTool {

    /**
     * 流程json
     */
    private Map<String, Object> flowDataMap;

    /**
     * 要发布到的项目信息
     */
    private DmpProject publishedToProject;

    /**
     * 要发布到的项目配置信息
     */
    private DmpProjectSystemInfo publishedToProjectSystemInfo;


    private DmpDevelopTaskService dmpDevelopTaskService;

    private DmpNavigationTreeService dmpNavigationTreeService;

    private DmpWorkFlowSubmitDetailsService dmpWorkFlowSubmitDetailsService;

    /**
     * 不发布项目用
     *
     * @param flowPro
     */
    public FlowParseTool(FlowPro flowPro, DmpWorkFlowSubmitDetailsService dmpWorkFlowSubmitDetailsService) {
        this.flowPro = flowPro;
        this.flowDataMap = JSON.parseObject(flowPro.getFlowJson(), Map.class);
        this.dmpWorkFlowSubmitDetailsService = dmpWorkFlowSubmitDetailsService;
        parse();
    }

    /**
     * 发布项目用
     *
     * @param flowPro
     * @param publishedToProject
     * @param publishedToProjectSystemInfo
     */
    public FlowParseTool(FlowPro flowPro,
                         DmpProject publishedToProject,
                         DmpProjectSystemInfo publishedToProjectSystemInfo,
                         DmpDevelopTaskService dmpDevelopTaskService,
                         DmpNavigationTreeService dmpNavigationTreeService,
                         DmpWorkFlowSubmitDetailsService dmpWorkFlowSubmitDetailsService) {
        this(flowPro, dmpWorkFlowSubmitDetailsService);
        this.publishedToProject = publishedToProject;
        this.publishedToProjectSystemInfo = publishedToProjectSystemInfo;
        this.dmpDevelopTaskService = dmpDevelopTaskService;
        this.dmpNavigationTreeService = dmpNavigationTreeService;
    }


    /**
     * 流程属性
     */
    private FlowPro flowPro;

    /**
     * 节点依赖关系
     */
    private Map<String, String> nodeDependcyRefMap;


    /**
     * 流程节点
     * key是节点id
     */
    private Map<String, FlowNode> flowNodeMap;

    /**
     * 流程变更数据
     */
    private Map flowChangedMap;


    private void parse() {

        //处理流程数据
        parseFlowPro();

        //处理节点关系
        parseEdges();

        //处理流程节点
        parseNodes();

        //处理依赖关系
        handleFlowNodeDependcy();

    }


    /**
     * 处理流程属性
     */
    private void parseFlowPro() {
        String key = "flowPro";
        Map<String, Object> flowProMap = (Map) flowDataMap.get(key);
        String flowName = flowProMap.get("flowName").toString();
        //String taskId = flowProMap.get("taskId");
        String scheduleSetting = flowProMap.get("scheduleSetting").toString();
        String afterFailSelect = flowProMap.get("afterFailSelect").toString();

        flowPro.setProjectId(Long.parseLong(flowProMap.get("projectId").toString()));
        flowPro.setTreeId(Long.parseLong(flowProMap.get("treeId").toString()));
        flowPro.setFlowName(flowName);
        //flowPro.setTaskId(taskId);
        flowPro.setScheduleSetting(scheduleSetting);
        flowPro.setAfterFailSelect(afterFailSelect);

    }


    /**
     * 处理线连接的节点
     */
    private void parseEdges() {
        String key = "edges";
        List<Map<String, String>> edgesRelationList = (List<Map<String, String>>) flowDataMap.get(key);
        int size = edgesRelationList.size();
        nodeDependcyRefMap = new HashMap<>(size);
        Map<String, String> edgesRelationMap;
        for (int i = 0; i < size; i++) {
            edgesRelationMap = edgesRelationList.get(i);
            String sourceNodeId = edgesRelationMap.get("source");
            String targetNodeId = edgesRelationMap.get("target");
            if (nodeDependcyRefMap.get(targetNodeId) != null) {
                //说明已经有
                nodeDependcyRefMap.put(targetNodeId, nodeDependcyRefMap.get(targetNodeId) + "," + sourceNodeId);
            } else {
                //没有  直接存
                nodeDependcyRefMap.put(targetNodeId, sourceNodeId);
            }

        }
    }

    /**
     * 流程节点处理
     *
     * @return
     */
    private void parseNodes() {
        String key = "nodes";
        List<Map<String, String>> nodesList = (List<Map<String, String>>) flowDataMap.get(key);
        int size = nodesList.size();
        flowNodeMap = new HashMap<>(size);
        FlowNode flowNode;
        Map<String, String> tempMap;

        for (int i = 0; i < size; i++) {
            tempMap = nodesList.get(i);
            String nodeId = tempMap.get("id");
            String nodeName = tempMap.get("label");
            String nodeType = tempMap.get("taskType");
            String nodeLocation = tempMap.get("nodeLocation");
            String script = tempMap.get("script");
            String retryTimes = tempMap.get("retryTimes");
            String retryInterval = tempMap.get("retryInterval");
            flowNode = new FlowNode();
            flowNode.setNodeId(nodeId);
            flowNode.setNodeName(nodeName);
            flowNode.setNodeType(nodeType);
            flowNode.setNodeLocation(nodeLocation);
            flowNode.setScript(script);
            flowNode.setRetryTimes(retryTimes);
            flowNode.setRetryInterval(retryInterval);
            flowNode.setNodeData(JSON.toJSONString(tempMap));
            flowNodeMap.put(nodeId, flowNode);
        }
    }

    /**
     * 处理流程节点依赖
     */
    private void handleFlowNodeDependcy() {

        Iterator<Map.Entry<String, FlowNode>> iterator = flowNodeMap.entrySet().iterator();
        StringBuffer sb;
        FlowNode flowNode;
        while (iterator.hasNext()) {
            Map.Entry<String, FlowNode> next = iterator.next();
            String nodeId = next.getKey();
            flowNode = next.getValue();

            String dependcyNodeIds = nodeDependcyRefMap.get(nodeId);

            if (StringUtils.isNotBlank(dependcyNodeIds)) {
                sb = new StringBuffer();
                String[] dependcyNodeIdArr = dependcyNodeIds.split(",");
                for (int i = 0; i < dependcyNodeIdArr.length; i++) {
                    String dependcyNodeId = dependcyNodeIdArr[i];
                    String dependcyNodeName = flowNodeMap.get(dependcyNodeId).getNodeName();
                    sb.append(dependcyNodeName);
                    sb.append(",");
                }
                flowNode.setDependedNodeName(sb.toString().substring(0, sb.toString().length() - 1));
            }
        }

    }


    /**
     * 获取节点变更列表
     * @return
     */
    public List<FlowNodeChangeInfo> getChangedNodes() {

        String currentUserName = "superadmin";
        String currentDateStr = DateFormatUtils.format(new Date(), "yyyy-MM-dd HH:mm:ss");


        Long projectId = flowPro.getProjectId();
        String flowName = flowPro.getFlowName();
        boolean isCheckVerion = flowPro.isCheckVerion();


        Map queryParamMap = new HashMap();
        queryParamMap.put("projectId", projectId);
        queryParamMap.put("flowName", flowName);
        queryParamMap.put("isCheckVerion", isCheckVerion);

        List<DmpWorkFlowSubmitDetails> queryList = dmpWorkFlowSubmitDetailsService.getLatestUnderFlowByNodeName(queryParamMap);
        Map<String, DmpWorkFlowSubmitDetails> queryMap = new HashMap<>();
        if (queryList != null && queryList.size() > 0) {
            for (DmpWorkFlowSubmitDetails dmpWorkFlowSubmitDetails : queryList) {
                queryMap.put(dmpWorkFlowSubmitDetails.getNodeName(), dmpWorkFlowSubmitDetails);
            }
        }



        //流程变更节点保存
        List<FlowNodeChangeInfo> flowNodeChangeList = new ArrayList<>();


        // 比较流程变更
        Iterator<Map.Entry<String, FlowNode>> iterator = flowNodeMap.entrySet().iterator();
        FlowNode flowNode = null;
        DmpWorkFlowSubmitDetails dmpWorkFlowSubmitDetails = null;
        FlowNodeChangeInfo flowNodeChangeInfo = null;
        while (iterator.hasNext()) {

            flowNode = iterator.next().getValue();
            String nodeName = flowNode.getNodeName();
            String nodeType = flowNode.getNodeType();
            String nodeData = flowNode.getNodeData();

            dmpWorkFlowSubmitDetails = queryMap.get(nodeName);
            if (dmpWorkFlowSubmitDetails == null) {
                //没保存过当前节点  所有节点就是新增
                flowNodeChangeInfo = new FlowNodeChangeInfo();
                flowNodeChangeInfo.setNodeName(nodeName);
                flowNodeChangeInfo.setNodeData(nodeData);
                flowNodeChangeInfo.setNodeType(nodeType);
                flowNodeChangeInfo.setChangedType(NodeChangeTypeEnum.ADD.getChangeType());
                flowNodeChangeInfo.setCreateBy(currentUserName);
                flowNodeChangeInfo.setUpdateDateStr(currentDateStr);
                flowNodeChangeInfo.setVersion(1);
                flowNodeChangeList.add(flowNodeChangeInfo);

            } else {
                //保存过当前节点 比较几点数据是否一致 不一致
                if (!nodeData.equals(dmpWorkFlowSubmitDetails.getNodeData())) {
                    flowNodeChangeInfo = new FlowNodeChangeInfo();
                    flowNodeChangeInfo.setNodeName(nodeName);
                    flowNodeChangeInfo.setNodeData(nodeData);
                    flowNodeChangeInfo.setNodeType(nodeType);
                    flowNodeChangeInfo.setChangedType(NodeChangeTypeEnum.UPDATE.getChangeType());
                    flowNodeChangeInfo.setCreateBy(currentUserName);
                    flowNodeChangeInfo.setUpdateDateStr(currentDateStr);
                    flowNodeChangeInfo.setVersion(dmpWorkFlowSubmitDetails.getVersion() == null ? 1 : dmpWorkFlowSubmitDetails.getVersion() + 1);
                    flowNodeChangeList.add(flowNodeChangeInfo);
                }

                //当前流程有的节点删掉   没有的就全部是删除的
                queryMap.remove(nodeName);
            }
        }

        // 获取删除的节点

        /*Iterator<Map.Entry<String, DmpWorkFlowSubmitDetails>> delIterator = queryMap.entrySet().iterator();
        while (delIterator.hasNext()) {
            dmpWorkFlowSubmitDetails = delIterator.next().getValue();

            String nodeName = dmpWorkFlowSubmitDetails.getNodeName();
            flowNodeChangeInfo = new FlowNodeChangeInfo();
            flowNodeChangeInfo.setNodeName(nodeName);
            flowNodeChangeInfo.setChangedType(NodeChangeTypeEnum.DELETE.getChangeType());
            flowNodeChangeInfo.setCreateBy(currentUserName);
            flowNodeChangeInfo.setUpdateDateStr(currentDateStr);
            flowNodeChangeInfo.setNodeData(dmpWorkFlowSubmitDetails.getNodeData());
            flowNodeChangeList.add(flowNodeChangeInfo);
        }*/
         return flowNodeChangeList;
    }


    /**
     * 发布流程
     */
    public boolean publish()throws Exception {

        Long publishedToProjectId = publishedToProjectSystemInfo.getProjectId();
        Long treeId = flowPro.getTreeId();

        /**
         * 当前任务生成文件存放根路径
         */
        String localTaskPath = publishedToProjectSystemInfo.getAzkabanLocalTaskFilePath() + "/" + publishedToProjectId + "/" + treeId;
        File localTaskFile = new File(localTaskPath);
        if (!localTaskFile.exists()) {
            localTaskFile.mkdirs();
        }

        String localTaskSourcePath = localTaskPath + "/source/";
        File localTaskSourceFile = new File(localTaskSourcePath);
        if (!localTaskSourceFile.exists()) {
            localTaskSourceFile.mkdirs();
        }

        String localTaskExecArgsPath = localTaskPath + "/execArgs/";
        File localTaskExecArgsFile = new File(localTaskExecArgsPath);
        if (!localTaskExecArgsFile.exists()) {
            localTaskExecArgsFile.mkdirs();
        }

        String localTaskZipPath = localTaskPath + "/target/";
        File localTaskZipFile = new File(localTaskZipPath);
        if (!localTaskZipFile.exists()) {
            localTaskZipFile.mkdirs();
        }

        //先删除上次创建的文件 ---每次都会新建
        FileUtils.deleteDirectory(localTaskPath);


        List<String> contents;

        Iterator<Map.Entry<String, FlowNode>> iterator = flowNodeMap.entrySet().iterator();
        String azkabanJobType = "";
        String azkabanJobCommand = "";
        while (iterator.hasNext()) {
            contents = new ArrayList<>();
            FlowNode flowNode = iterator.next().getValue();
            String nodeType = flowNode.getNodeType();
            if ("shell".equalsIgnoreCase(nodeType)) {
                // shell
                azkabanJobType = "command";
                azkabanJobCommand = generateShellFile(flowNode, localTaskExecArgsPath);

            } else if ("sql".equalsIgnoreCase(nodeType)) {
                // sql 任务
                azkabanJobType = "command";
                azkabanJobCommand = generateSqlFile(flowNode, localTaskExecArgsPath);

            } else if ("sync".equalsIgnoreCase(nodeType)) {
                //同步任务
                azkabanJobType = "command";
                azkabanJobCommand = generateSyncFile(flowNode);

            } else if ("subprocess".equalsIgnoreCase(nodeType)) {
                //子流程
                azkabanJobType = "flow";
                azkabanJobCommand = generateSubprocessFile(flowNode);
            }

            //子流程类型
            contents.add("type=" + azkabanJobType);
            contents.add(azkabanJobCommand);

            String dependenciesNodeName = flowNode.getDependedNodeName();
            if (StringUtils.isNotBlank(dependenciesNodeName)) {
                contents.add("dependencies=" + dependenciesNodeName);
            }

            String retrytimes = flowNode.getRetryTimes();
            if (StringUtils.isNotBlank(retrytimes)) {
                contents.add("retries=" + retrytimes);
            }

            String retrybackoff = flowNode.getRetryInterval();
            if (StringUtils.isNotBlank(retrybackoff)) {
                contents.add("retry.backoff=" + retrybackoff);
            }
            //生成job文件
            String jobFileAbsolutePath = localTaskSourcePath + flowNode.getNodeName() + ".job";
            FileUtils.write(jobFileAbsolutePath, contents);
        }

        String localZipTargetFileName = flowPro.getFlowName() + ".zip";
        ZipUtils.zip(localTaskSourcePath, localTaskZipPath, localZipTargetFileName);

        //上传到azkaban todo
        //上次zip包到azkaban
        String localTaskZipAbsolutePath = localTaskZipPath + "/" + localZipTargetFileName;
        String azkabanApiUrl = publishedToProjectSystemInfo.getAzkabanMonitorUrl();
        AzkabanApiUtils2 azkabanApiUtils = new AzkabanApiUtils2(azkabanApiUrl);
        return azkabanApiUtils.loginCreateProjectuploadZipAndSchedule("jz_workflow_" + publishedToProjectId, publishedToProject.getProjectDesc(), localTaskZipAbsolutePath, flowPro);
    }


    /**
     * 生成shell 文件
     *
     * @param flowNode
     * @return
     */
    private String generateShellFile(FlowNode flowNode, String localTaskExecArgsPath) {

        String fileName = flowNode.getNodeName() + ".sh";
        String scriptFileAbsolutePath = localTaskExecArgsPath + fileName;
        Long publishedToProjectId = publishedToProjectSystemInfo.getProjectId();
        Long treeId = flowPro.getTreeId();

        List<String> list = new ArrayList<>();
        list.add("#!/bin/sh");
        list.add(flowNode.getScript());
        FileUtils.write(scriptFileAbsolutePath, list);


        //远程shell 路径
        String remoteShellDir = publishedToProjectSystemInfo.getAzkabanExectorShellPath() + "/" + publishedToProjectId + "/" + treeId + "/";
        //上传shell文件  todo
        SFTPUtils sftpUtils = new SFTPUtils(publishedToProjectSystemInfo.getShellCmdServer(),
                publishedToProjectSystemInfo.getShellCmdUser(),
                publishedToProjectSystemInfo.getShellCmdPassword(),
                publishedToProjectSystemInfo.getShellSftpPort());
        sftpUtils.singleUploadFile(localTaskExecArgsPath, fileName, remoteShellDir);


        String command = "command=" + publishedToProjectSystemInfo.getAzkabanExectorShellExec() + " " + publishedToProjectId + " ${azkaban.flow.flowid} ${azkaban.job.id} " + remoteShellDir + fileName;
        return command;
    }

    /**
     * 生成sql文件
     *
     * @param flowNode
     * @param localTaskExecArgsPath
     * @return
     */
    private String generateSqlFile(FlowNode flowNode, String localTaskExecArgsPath) {

        String fileName = flowNode.getNodeName() + ".sql";
        String scriptFileAbsolutePath = localTaskExecArgsPath + fileName;
        Long publishedToProjectId = publishedToProjectSystemInfo.getProjectId();
        Long treeId = flowPro.getTreeId();
        FileUtils.write(scriptFileAbsolutePath, flowNode.getScript());

        //上传sql文件  todo

        //远程shell 路径
        String remoteSqlDir = publishedToProjectSystemInfo.getAzkabanExectorSqlPath() + "/" + publishedToProjectId + "/" + treeId + "/";
        //上传shell文件  todo
        SFTPUtils sftpUtils = new SFTPUtils(publishedToProjectSystemInfo.getShellCmdServer(),
                publishedToProjectSystemInfo.getShellCmdUser(),
                publishedToProjectSystemInfo.getShellCmdPassword(),
                publishedToProjectSystemInfo.getShellSftpPort());
        sftpUtils.singleUploadFile(localTaskExecArgsPath, fileName, remoteSqlDir);

        String command = "command=" + publishedToProjectSystemInfo.getAzkabanExectorSqlExec() + " " + publishedToProjectId + " ${azkaban.flow.flowid} ${azkaban.job.id} " + remoteSqlDir + fileName;
        return command;
    }

    /**
     * 生成同步任务文件
     *
     * @param flowNode
     * @return
     */
    private String generateSyncFile(FlowNode flowNode)throws Exception {
        /*Long syncTaskId = job.getXmlTaskTreeId();
        if (syncTaskId == null) {
            throw new RuntimeException("请选择同步任务节点");
        }
        String execXmlFileNameAndVersion = getPublishSyncTaskFileNameAndLatestVersion(syncTaskId);
        job.setXmlSyncTaskFileNameAndLatestVersion(execXmlFileNameAndVersion);

        String execXmlFileName = execXmlFileNameAndVersion.split("@")[1];*/
        //任务所在项目id
        Long projectId = flowPro.getProjectId();
        Long publishedToProjectId = publishedToProjectSystemInfo.getProjectId();

        //根据taskName获取treeId
        String taskName = flowNode.getScript();

        DmpNavigationTree dmpNavigationTree = dmpNavigationTreeService.getByProjectIdAndName(projectId, taskName);
        if (dmpNavigationTree == null) {
            throw new RuntimeException("未找到任务:" + taskName);
        }
        Long syncTaskTreeId = dmpNavigationTree.getId().longValue();

        //暂时不上传
        //dmpDevelopTaskService.newPublishSyncing(syncTaskTreeId, publishedToProjectId);
        //获取最新版本的同步任务
        String execXmlFileNameAndVersion = getPublishSyncTaskFileNameAndLatestVersion(taskName, syncTaskTreeId);
        String execXmlFileName = execXmlFileNameAndVersion.split("@")[1];
        //xml  执行xml的命令写到job文件中
        String command = "command=" + publishedToProjectSystemInfo.getAzkabanExectorXmlExec() + " " + publishedToProjectId + " ${azkaban.flow.flowid} ${azkaban.job.id} " + execXmlFileName;
        return command;
    }


    /**
     * 获取选择的数据同步的最新版本
     *
     * @param syncTaskTreeId 数据同步任务treeId
     * @return
     */
    private String getPublishSyncTaskFileNameAndLatestVersion(String syncTaskName, Long syncTaskTreeId)throws Exception {

        String execXmlFileNameAndVersion = dmpDevelopTaskService.getConfigFileNameNotSuffix4Published(syncTaskTreeId);
        if (org.springframework.util.StringUtils.isEmpty(execXmlFileNameAndVersion)) {
            throw new RuntimeException("同步任务:" + syncTaskName + " xml配置文件为空,请确认是否提交成功");
        }
        return syncTaskTreeId + "@" + execXmlFileNameAndVersion;
    }

    /**
     * 生成子流程
     *
     * @param flowNode
     * @return
     */
    private String generateSubprocessFile(FlowNode flowNode) {

        String subProcessFlowName = flowNode.getScript();
        //检查子流程是否存在 todo
        String azkabanApiUrl = publishedToProjectSystemInfo.getAzkabanMonitorUrl();
        AzkabanApiUtils2 azkabanApiUtils = new AzkabanApiUtils2(azkabanApiUrl);
        boolean flowExists = azkabanApiUtils.checkFlowExists("jz_workflow_" + flowPro.getPublishedToProjectId(), subProcessFlowName);
        if (!flowExists) {
            throw new RuntimeException("节点:" + flowNode.getNodeName() + "设置的子流程:" + subProcessFlowName + "不存在,请先发布" + subProcessFlowName);
        }
        return "flow.name=" + subProcessFlowName;
    }


}
