package com.jz.dmp.modules.controller.dataService;

import com.jz.common.constant.JsonResult;
import com.jz.common.constant.ResultCode;
import com.jz.dmp.modules.controller.dataOperation.bean.DataDevTaskListDto;
import com.jz.dmp.modules.controller.dataService.bean.OrganizationManageAddReq;
import com.jz.dmp.modules.controller.dataService.bean.OrganizationManageListQueryReq;
import com.jz.dmp.modules.controller.dataService.bean.OrganizationManageUpdateReq;
import com.jz.dmp.modules.service.DmpOrgMangeService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;

/**
 * 数据服务组织管理--调用gateway 接口
 *
 * @author Bellamy
 * @since 2020-12-24 10:56:18
 */
@RestController
@RequestMapping("/orgMange")
@Api(tags = "数据服务-组织管理")
public class DmpOrgMangeController {

    private static Logger logger = LoggerFactory.getLogger(DmpOrgMangeController.class);
    /**
     * 服务对象
     */
    @Autowired
    private DmpOrgMangeService dmpOrgMangeService;

    /**
     * 列表分页查询
     *
     * @return
     * @author Bellamy
     * @since 2021-01-16
     */
    @ApiOperation(value = "列表分页查询", notes = "列表分页查询")
    @PostMapping(value = "/listPage")
    public JsonResult getOrgListPage(@RequestBody @Validated OrganizationManageListQueryReq req, HttpServletRequest httpRequest) {
        JsonResult jsonResult = new JsonResult();
        try {
            jsonResult = dmpOrgMangeService.queryOrgListPage(req);
        } catch (Exception e) {
            jsonResult.setMessage(e.getMessage());
            jsonResult.setCode(ResultCode.INTERNAL_SERVER_ERROR);
            e.printStackTrace();
        }
        return jsonResult;
    }

    /**
     * 删除组织
     *
     * @return
     * @author Bellamy
     * @since 2021-01-16
     */
    @ApiOperation(value = "删除组织", notes = "删除组织")
    @GetMapping(value = "/delOrg")
    @ApiImplicitParam(name = "id", value = "组织id", required = true)
    public JsonResult delOrgById(@RequestParam long id, HttpServletRequest httpRequest) {
        JsonResult jsonResult = new JsonResult();
        try {
            jsonResult = dmpOrgMangeService.delOrgById(id);
        } catch (Exception e) {
            jsonResult.setMessage(e.getMessage());
            jsonResult.setCode(ResultCode.INTERNAL_SERVER_ERROR);
            e.printStackTrace();
        }
        return jsonResult;
    }

    /**
     * 新增组织
     *
     * @return
     * @author Bellamy
     * @since 2021-01-16
     */
    @ApiOperation(value = "新增组织", notes = "新增组织")
    @PostMapping(value = "/addOrg")
    public JsonResult addOrg(@RequestBody @Validated OrganizationManageAddReq req, HttpServletRequest httpRequest) {
        if (StringUtils.isEmpty(req.getOrgName())) {
            return JsonResult.error(ResultCode.PARAMS_ERROR, "组织名称不能为空!");
        }
        if (StringUtils.isEmpty(req.getOrgType())) {
            return JsonResult.error(ResultCode.PARAMS_ERROR, "组织类型不能为空!");
        }
        JsonResult jsonResult = new JsonResult();
        try {
            jsonResult = dmpOrgMangeService.addOrg(req);
        } catch (Exception e) {
            jsonResult.setMessage(e.getMessage());
            jsonResult.setCode(ResultCode.INTERNAL_SERVER_ERROR);
            e.printStackTrace();
        }
        return jsonResult;
    }

    /**
     * 编辑组织
     *
     * @return
     * @author Bellamy
     * @since 2021-01-16
     */
    @ApiOperation(value = "编辑组织", notes = "编辑组织")
    @PostMapping(value = "/updateOrg")
    public JsonResult updateOrg(@RequestBody @Validated OrganizationManageUpdateReq req, HttpServletRequest httpRequest) {
        JsonResult jsonResult = new JsonResult();
        try {
            jsonResult = dmpOrgMangeService.updateOrg(req);
        } catch (Exception e) {
            jsonResult.setMessage(e.getMessage());
            jsonResult.setCode(ResultCode.INTERNAL_SERVER_ERROR);
            e.printStackTrace();
        }
        return jsonResult;
    }

    /**
     * 根据组织id获取组织详情
     *
     * @author Bellamy
     * @since 2021-01-18
     */
    @ApiOperation(value = "根据组织id获取组织详情", notes = "根据组织id获取组织详情")
    @GetMapping(value = "/orgInfo")
    @ApiImplicitParam(name = "id", value = "组织id", required = true)
    public JsonResult getOrgInfoByOrgId(@RequestParam String id, HttpServletRequest httpRequest) {
        JsonResult jsonResult = new JsonResult();
        if (StringUtils.isEmpty(id)) {
            return JsonResult.error(ResultCode.PARAMS_ERROR, "组织id不能为空！");
        }
        try {
            jsonResult = dmpOrgMangeService.getOrgInfoByOrgId(id);
        } catch (Exception e) {
            jsonResult.setMessage(e.getMessage());
            jsonResult.setCode(ResultCode.INTERNAL_SERVER_ERROR);
            e.printStackTrace();
        }
        return jsonResult;
    }
}