package com.jz.dmp.modules.service;

import com.jz.common.constant.JsonResult;
import com.jz.common.page.PageInfoResponse;
import com.jz.dmp.modules.controller.DataIntegration.bean.*;
import com.jz.dmp.modules.model.DmpRealtimeSyncInfo;
import com.jz.dmp.modules.model.RealTimeSyncModel;

import java.util.List;
import java.util.Map;

/**
 * 实时同步任务(DmpRealtimeSyncInfo)表服务接口
 *
 * @author Bellamy
 * @since 2021-01-05 14:18:03
 */
public interface DmpRealtimeSyncInfoService {

    /**
     * 通过ID查询单条数据
     *
     * @param id 主键
     * @return 实例对象
     */
    DmpRealtimeSyncInfo queryById(Integer id);

    /**
     * 查询多条数据
     *
     * @param offset 查询起始位置
     * @param limit  查询条数
     * @return 对象列表
     */
    List<DmpRealtimeSyncInfo> queryAllByLimit(int offset, int limit);

    /**
     * 新增数据
     *
     * @param dmpRealtimeSyncInfo 实例对象
     * @return 实例对象
     */
    DmpRealtimeSyncInfo insert(DmpRealtimeSyncInfo dmpRealtimeSyncInfo);

    /**
     * 修改数据
     *
     * @param dmpRealtimeSyncInfo 实例对象
     * @return 实例对象
     */
    DmpRealtimeSyncInfo update(DmpRealtimeSyncInfo dmpRealtimeSyncInfo);

    /**
     * 通过主键删除数据
     *
     * @param id 主键
     * @return 是否成功
     */
    boolean deleteById(Integer id);

    /**
     * 实时同步任务列表分页查询
     *
     * @return
     */
    PageInfoResponse<RealTimeSyncListDto> queryRealTimeSyncListPage(RealTimeSyncListReq req) throws Exception;

    /**
     * 根据多个实时任务id查询任务信息
     *
     * @return
     */
    List<DmpRealtimeSyncInfo> queryListById(String[] ids) throws Exception;

    /**
     * 获取数据源和目标数据源下拉框
     *
     * @return
     * @author Bellamy
     * @since 2021-01-06
     */
    JsonResult<List<DataSourceNameListDto>> queryDatasourceNameList(String projectId, String type) throws Exception;

    /**
     * 新增--选择源表信息
     *
     * @return
     * @author Bellamy
     * @since 2021-01-06
     */
    JsonResult<RealTimeSyncModel> queryTableInfoByParams(RealTimeTableInfoReq req) throws Exception;

    /**
     * 新增--选择目标信息
     *
     * @return
     * @author Bellamy
     * @since 2021-01-07
     */
    JsonResult selectTargetDatasourceInfo(String projectId) throws Exception;

    JsonResult queryRealTimeInfoByDataSourceId(String srcDatasourceId, String sourceTableName) throws Exception;

    /**
     * 保存实时同步任务
     *
     * @return
     * @author Bellamy
     * @since 2021-01-08
     */
    JsonResult addRealTimeTask(Map<String, Object> params) throws Exception;

    /**
     * 编辑实时同步任务
     *
     * @return
     * @author Bellamy
     * @since 2021-01-08
     */
    JsonResult updateRealTimeTask(Map<String, Object> params) throws Exception;

    /**
     * 编辑--数据回显,通过id查询任务相关信息
     *
     * @return
     * @author Bellamy
     * @since 2021-01-12
     */
    JsonResult<RealTimeEditDataEchoDto> selectRealtimeTaskById(String taskId) throws Exception;

    /**
     * 批量删除数据源
     *
     * @return
     * @author Bellamy
     */
    boolean deleteByrealTaskId(String realTaskId) throws Exception;

    /**
     * 批量上下线
     *
     * @return
     * @author Bellamy
     * @since 2021-01-05
     */
    boolean batchUptOnlineStatus(String realTaskId, String onlineStatus) throws Exception;
}