package com.jz.dmp.modules.service.sys.impl;

import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import javax.servlet.http.HttpServletRequest;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.jz.common.bean.BaseBeanResponse;
import com.jz.common.bean.BaseResponse;
import com.jz.common.bean.PageInfoResponse;
import com.jz.common.constant.Constants;
import com.jz.common.constant.ResultCode;
import com.jz.common.persistence.BaseService;
import com.jz.dmp.modules.controller.sys.bean.DmpProjectRoleBatch;
import com.jz.dmp.modules.controller.sys.bean.DmpProjectRoleDto;
import com.jz.dmp.modules.controller.sys.bean.DmpProjectRoleRequest;
import com.jz.dmp.modules.dao.sys.DmpProjectRoleMapper;
import com.jz.dmp.modules.model.DmpProjectRole;
import com.jz.dmp.modules.service.sys.DmpProjectRoleService;

/**
 * 项目角色关系表服务的实现?
 * @author ybz
 *
 */
@Service("dmpProjectRoleService")
public class DmpProjectRoleServiceImpl extends BaseService implements DmpProjectRoleService {
	
	private static Logger logger = LoggerFactory.getLogger(DmpProjectRoleServiceImpl.class);
	
	@Autowired
	private DmpProjectRoleMapper dmpProjectRoleMapper;

	/* (non-Javadoc)
	 * @see com.ycxc.vmts.service.DmpProjectRoleService#findList(com.ycxc.vmts.controller.bean.DmpProjectRoleRequest, javax.servlet.http.HttpServletRequest)
	 */
	@Override
	public PageInfoResponse<DmpProjectRoleDto> findList(DmpProjectRoleRequest dmpProjectRoleRequest, HttpServletRequest httpRequest)
			throws Exception {
		PageInfoResponse<DmpProjectRoleDto> pageInfoResponse = new PageInfoResponse<>();
		
		Map<String, Object> param = new HashMap<>();
		//主键
		if (dmpProjectRoleRequest.getProjectRoleId() != null) {
			param.put("projectRoleId", dmpProjectRoleRequest.getProjectRoleId());
		}
		//项目ID
		if (dmpProjectRoleRequest.getProjectId() != null) {
			param.put("projectId", dmpProjectRoleRequest.getProjectId());
		}
		//角色ID
		if (dmpProjectRoleRequest.getRoleId() != null) {
			param.put("roleId", dmpProjectRoleRequest.getRoleId());
		}
		
		PageHelper.startPage(dmpProjectRoleRequest.getPageNum(), dmpProjectRoleRequest.getPageSize());
		List<DmpProjectRoleDto> list = dmpProjectRoleMapper.findList(param);
		PageInfo<DmpProjectRoleDto> pageInfo = new PageInfo<>(list);
		
		pageInfoResponse.setCode(ResultCode.SUCCESS.val());
		pageInfoResponse.setMessage("查询成功");
		pageInfoResponse.setData(pageInfo);
		return pageInfoResponse;
	}
	

	/* (non-Javadoc)
	 * @see com.ycxc.vmts.service.DmpProjectRoleService#add(com.ycxc.vmts.entity.DmpProjectRole, javax.servlet.http.HttpServletRequest)
	 */
	@Override
	@Transactional(rollbackFor=Exception.class)
	public BaseBeanResponse<DmpProjectRole> add(DmpProjectRole dmpProjectRole, HttpServletRequest httpRequest) throws Exception {
		BaseBeanResponse<DmpProjectRole> baseBeanResponse = new BaseBeanResponse<>();
		
		dmpProjectRoleMapper.insertSelective(dmpProjectRole);
		
		baseBeanResponse.setCode(ResultCode.SUCCESS.val());
		baseBeanResponse.setMessage("新增成功");
		baseBeanResponse.setData(dmpProjectRole);
		
		return baseBeanResponse;
	}

	/* (non-Javadoc)
	 * @see com.ycxc.vmts.service.DmpProjectRoleService#edit(com.ycxc.vmts.entity.DmpProjectRole, javax.servlet.http.HttpServletRequest)
	 */
	@Override
	@Transactional(rollbackFor=Exception.class)
	public BaseBeanResponse<DmpProjectRole> edit(DmpProjectRole dmpProjectRole, HttpServletRequest httpRequest) throws Exception {
		BaseBeanResponse<DmpProjectRole> baseBeanResponse = new BaseBeanResponse<>();
		
		dmpProjectRoleMapper.updateByPrimaryKeySelective(dmpProjectRole);
		
		baseBeanResponse.setCode(ResultCode.SUCCESS.val());
		baseBeanResponse.setMessage("修改成功");
		baseBeanResponse.setData(dmpProjectRole);
		
		return baseBeanResponse;
	}

	/* (non-Javadoc)
	 * @see com.ycxc.vmts.service.DmpProjectRoleService#findById(com.ycxc.vmts.controller.bean.DmpProjectRoleRequest, javax.servlet.http.HttpServletRequest)
	 */
	@Override
	public BaseBeanResponse<DmpProjectRoleDto> findById(DmpProjectRoleRequest dmpProjectRoleRequest, HttpServletRequest httpRequest)
			throws Exception {
		BaseBeanResponse<DmpProjectRoleDto> baseBeanResponse = new BaseBeanResponse<>();
		
		DmpProjectRoleDto dmpProjectRoleDto = dmpProjectRoleMapper.findById(dmpProjectRoleRequest.getProjectRoleId());
		
		baseBeanResponse.setCode(ResultCode.SUCCESS.val());
		baseBeanResponse.setMessage("查询成功");
		baseBeanResponse.setData(dmpProjectRoleDto);
		
		return baseBeanResponse;
	}

	/* (non-Javadoc)
	 * @see com.ycxc.vmts.service.DmpProjectRoleService#deleteById(com.ycxc.vmts.controller.bean.DmpProjectRoleRequest, javax.servlet.http.HttpServletRequest)
	 */
	@Override
	@Transactional(rollbackFor=Exception.class)
	public BaseResponse deleteById(DmpProjectRoleRequest dmpProjectRoleRequest, HttpServletRequest httpRequest)
			throws Exception {
		BaseResponse baseResponse = new BaseResponse();
		
		dmpProjectRoleMapper.deleteByPrimaryKey(dmpProjectRoleRequest.getProjectRoleId());
		
		baseResponse.setCode(ResultCode.SUCCESS.val());
		baseResponse.setMessage("删除成功");
		
		return baseResponse;
	}

	/* (non-Javadoc)
	 * @see com.ycxc.vmts.service.DmpProjectRoleService#addBatch(com.ycxc.vmts.controller.bean.DmpProjectRoleVo, javax.servlet.http.HttpServletRequest)
	 */
	@Override
	@Transactional(rollbackFor=Exception.class)
	public BaseBeanResponse<DmpProjectRoleBatch> addBatch(DmpProjectRoleBatch dmpProjectRoleBatch, HttpServletRequest httpRequest) throws Exception {
		BaseBeanResponse<DmpProjectRoleBatch> baseBeanResponse = new BaseBeanResponse<>();
		
		List<DmpProjectRole> dmpProjectRoles = dmpProjectRoleBatch.getDmpProjectRoles();
		
		if (CollectionUtils.isEmpty(dmpProjectRoles)) {
			baseBeanResponse.setCode(ResultCode.PARAMS_ERROR.val());
			baseBeanResponse.setMessage("数据不能为空！");
			return baseBeanResponse;
		}
		
		dmpProjectRoles = dmpProjectRoles.stream().map(dmpProjectRole -> {
			return dmpProjectRole;
		}).collect(Collectors.toList());
		
		dmpProjectRoleMapper.insertBatch(dmpProjectRoles);
		
		baseBeanResponse.setCode(ResultCode.SUCCESS.val());
		baseBeanResponse.setMessage("批量新增成功");
		baseBeanResponse.setData(dmpProjectRoleBatch);
		
		return baseBeanResponse;
	}
	
}
